import * as React from "react";
import { StyleProp, View, ViewStyle } from "react-native";
import Surface from "../Surface";
import { DefaultTheme } from "styled-components";
declare type Props = React.ComponentProps<typeof Surface> & {
    /**
     * Resting elevation of the card which controls the drop shadow.
     */
    elevation?: number;
    /**
     * Function to execute on long press.
     */
    onLongPress?: () => void;
    /**
     * Function to execute on press.
     */
    onPress?: () => void;
    /**
     * Content of the `Card`.
     */
    children: React.ReactNode;
    style?: StyleProp<ViewStyle>;
    /**
     * @optional
     */
    theme?: DefaultTheme;
    /**
     * Pass down testID from card props to touchable
     */
    testID?: string;
    /**
     * Pass down accessible from card props to touchable
     */
    accessible?: boolean;
};
/**
 * A card is a sheet of material that serves as an entry point to more detailed information.
 *
 * <div class="screenshots">
 *   <img class="medium" src="screenshots/card-1.png" />
 *   <img class="medium" src="screenshots/card-2.png" />
 * </div>
 *
 * ## Usage
 * ```js
 * import * as React from 'react';
 * import { AvatarIcon } from 'react-native-simple-elements/components/Avatar';
 * import Button from 'react-native-simple-elements/components/Button';
 * import Card from 'react-native-simple-elements/components/Card';
 * import { Title, Paragraph } from 'react-native-simple-elements/components/Text';
 *
 * const LeftContent = props => <AvatarIcon {...props} icon="folder" />
 *
 * const MyComponent = () => (
 *   <Card>
 *     <Card.Title title="Card Title" subtitle="Card Subtitle" left={LeftContent} />
 *     <Card.Content>
 *       <Title>Card title</Title>
 *       <Paragraph>Card content</Paragraph>
 *     </Card.Content>
 *     <Card.Cover source={{ uri: 'https://picsum.photos/700' }} />
 *     <Card.Actions>
 *       <Button>Cancel</Button>
 *       <Button>Ok</Button>
 *     </Card.Actions>
 *   </Card>
 * );
 *
 * export default MyComponent;
 * ```
 */
declare const Card: {
    ({ elevation: cardElevation, onLongPress, onPress, children, style, testID, accessible, ...rest }: Props): JSX.Element;
    Content: {
        ({ index, total, siblings, style, ...rest }: import("react-native").ViewProps & React.RefAttributes<View> & {
            children: React.ReactNode;
            index?: number;
            total?: number;
            siblings?: string[];
            style?: StyleProp<ViewStyle>;
        }): JSX.Element;
        displayName: string;
    };
    Actions: {
        (props: import("react-native").ViewProps & React.RefAttributes<View> & {
            children: React.ReactNode;
            style?: StyleProp<ViewStyle>;
        }): JSX.Element;
        displayName: string; /**
         * A card is a sheet of material that serves as an entry point to more detailed information.
         *
         * <div class="screenshots">
         *   <img class="medium" src="screenshots/card-1.png" />
         *   <img class="medium" src="screenshots/card-2.png" />
         * </div>
         *
         * ## Usage
         * ```js
         * import * as React from 'react';
         * import { AvatarIcon } from 'react-native-simple-elements/components/Avatar';
         * import Button from 'react-native-simple-elements/components/Button';
         * import Card from 'react-native-simple-elements/components/Card';
         * import { Title, Paragraph } from 'react-native-simple-elements/components/Text';
         *
         * const LeftContent = props => <AvatarIcon {...props} icon="folder" />
         *
         * const MyComponent = () => (
         *   <Card>
         *     <Card.Title title="Card Title" subtitle="Card Subtitle" left={LeftContent} />
         *     <Card.Content>
         *       <Title>Card title</Title>
         *       <Paragraph>Card content</Paragraph>
         *     </Card.Content>
         *     <Card.Cover source={{ uri: 'https://picsum.photos/700' }} />
         *     <Card.Actions>
         *       <Button>Cancel</Button>
         *       <Button>Ok</Button>
         *     </Card.Actions>
         *   </Card>
         * );
         *
         * export default MyComponent;
         * ```
         */
    };
    Cover: {
        ({ index, total, style, ...rest }: import("react-native").ImageProps & React.RefAttributes<import("react-native").Image> & {
            index?: number;
            total?: number;
            style?: StyleProp<ViewStyle>;
            theme?: DefaultTheme;
        }): JSX.Element;
        displayName: string;
    };
    Title: {
        ({ title, titleStyle, titleNumberOfLines, subtitle, subtitleStyle, subtitleNumberOfLines, left, leftStyle, right, rightStyle, style, }: import("react-native").ViewProps & React.RefAttributes<View> & {
            title: React.ReactNode;
            titleStyle?: StyleProp<import("react-native").TextStyle>;
            titleNumberOfLines?: number;
            subtitle?: React.ReactNode;
            subtitleStyle?: StyleProp<import("react-native").TextStyle>;
            subtitleNumberOfLines?: number;
            left?: (props: {
                size: number;
            }) => React.ReactNode; /**
             * Pass down testID from card props to touchable
             */
            leftStyle?: StyleProp<ViewStyle>;
            right?: (props: {
                size: number;
            }) => React.ReactNode;
            rightStyle?: StyleProp<ViewStyle>;
            index?: number;
            total?: number;
            style?: StyleProp<ViewStyle>;
            theme?: DefaultTheme;
        }): JSX.Element;
        displayName: string;
    };
};
export default Card;
